
from delivery.settings import DeliverySettings


class OrdersSettings(DeliverySettings):

    @property
    def INSTALLED_APPS(self):
        return super().INSTALLED_APPS + [
            'orders'
        ]


default = OrdersSettings




from django.contrib import admin
from django.utils.translation import ugettext_lazy as _

from cap.decorators import template_list_item, short_description

from orders.constants import DAYS_OF_WEEK
from orders.models import Order


@admin.register(Order)
class OrderAdmin(admin.ModelAdmin):

    list_display = [
        'order_tag', 'customer_tag', 'delivery_tag', 'status', 'products_tag']

    list_editable = ['status']

    list_filter = ['status']

    search_fields = [
        'first_name', 'last_name', 'middle_name', 'address', 'mobile', 'email',
        'comment'
    ]

    def has_add_permission(self, request):
        return False

    @template_list_item('orders/admin/customer.html', _('Customer'))
    def customer_tag(self, obj):
        return {'object': obj}

    @template_list_item('orders/admin/delivery.html', _('Delivery'))
    def delivery_tag(self, obj):
        return {'object': obj}

    @template_list_item('orders/admin/products.html', _('Products'))
    def products_tag(self, obj):
        return {'object': obj}

    @short_description(_('Order'))
    def order_tag(self, obj):
        day = DAYS_OF_WEEK[obj.created.weekday()]
        return '#{} - ({}) {}'.format(
            obj.id, day, obj.created.strftime('%d.%m.%Y %H:%M '))




from django.conf import settings
from django.contrib import messages
from django.template.loader import render_to_string
from django.contrib.sites.models import Site
from django.core.mail import mail_managers
from django.utils.safestring import mark_safe
from django.utils.translation import ugettext_lazy as _


def get_new_order_context(order):
    return {
        'order': order,
        'title': '{} #{}'.format(_('New order'), order.id),
        'site': Site.objects.get_current()
    }


def send_new_order_sms(context):
    try:
        from turbosms.lib import send_sms_from_template
        send_sms_from_template('orders/new_order_sms_for_manager.txt', context)
    except ImportError:
        pass
    except Exception as e:
        if settings.DEBUG:
            raise Exception('SMS sending error: {}'.format(e))


def send_new_order_email(context, subject=None):

    if subject is None:
        subject = context.get('title', '')

    html = render_to_string('orders/new_order_email_for_manager.html', context)

    mail_managers(subject=subject, message='', html_message=html)


def push_new_order_message_to_session(request, context):

    message = render_to_string('orders/new_order_message.html', context)

    messages.success(request, mark_safe(message))


def send_new_order_notifications(request, order):

    context = get_new_order_context(order)

    send_new_order_email(context)
    send_new_order_sms(context)
    push_new_order_message_to_session(request, context)



# Generated by Django 3.0.10 on 2020-09-08 12:57

import delivery.models
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('delivery', '0001_initial'),
        ('products', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='Order',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('status', models.CharField(choices=[('new', 'New order'), ('in_progress', 'In progress'), ('completed', 'Completed'), ('cancelled', 'Cancelled')], default='new', max_length=100, verbose_name='Status')),
                ('payment_method', models.CharField(choices=[('cash', 'Cash payment'), ('privat24', 'Privat 24'), ('cash_on_delivery', 'Cash on delivery'), ('cashless_payment', 'Cashless payment')], max_length=100, verbose_name='Payment method')),
                ('first_name', models.CharField(max_length=255, verbose_name='First name')),
                ('last_name', models.CharField(max_length=255, verbose_name='Last name')),
                ('middle_name', models.CharField(blank=True, max_length=255, verbose_name='Middle name')),
                ('address', models.CharField(blank=True, max_length=255, verbose_name='Address')),
                ('mobile', models.CharField(max_length=255, verbose_name='Mobile number')),
                ('created', models.DateTimeField(auto_now_add=True, verbose_name='Date created')),
                ('comment', models.TextField(blank=True, max_length=1000, verbose_name='Comment')),
                ('delivery', delivery.models.DeliveryMethodField(null=True, on_delete=django.db.models.deletion.CASCADE, to='delivery.DeliveryMethod', verbose_name='Delivery method')),
            ],
            options={
                'verbose_name': 'Order',
                'verbose_name_plural': 'Orders',
            },
        ),
        migrations.CreateModel(
            name='OrderedProduct',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('price_retail', models.FloatField(verbose_name='Retail price')),
                ('price_wholesale', models.FloatField(default=0, verbose_name='Wholesale price')),
                ('price_usd', models.FloatField(default=0)),
                ('price_eur', models.FloatField(default=0)),
                ('price_uah', models.FloatField(default=0)),
                ('initial_currency', models.PositiveIntegerField(choices=[(980, 'UAH'), (840, 'USD'), (978, 'EUR')], default=980, verbose_name='Currency')),
                ('qty', models.PositiveIntegerField(default=1, verbose_name='Quantity')),
                ('order', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='items', to='orders.Order', verbose_name='Order')),
                ('product', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='order_items', to='products.Product', verbose_name='Product')),
            ],
            options={
                'verbose_name': 'Ordered product',
                'verbose_name_plural': 'Ordered products',
            },
        ),
    ]




from django import forms

from orders.models import Order

from delivery.fields import DeliveryFormField


class CheckoutForm(forms.ModelForm):

    delivery = DeliveryFormField()

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        self.fields['delivery'].init_form(*args, **kwargs)

    class Meta:
        model = Order
        fields = [
            'first_name', 'last_name', 'middle_name', 'payment_method',
            'mobile', 'comment'
        ]




from django.utils.translation import ugettext_lazy as _


DAYS_OF_WEEK = [
    _('Monday'),
    _('Tuesday'),
    _('Wednesday'),
    _('Thursday'),
    _('Friday'),
    _('Saturday'),
    _('Sunday')
]

ORDER_STATUS_NEW = 'new'
ORDER_STATUS_IN_PROGRESS = 'in_progress'
ORDER_STATUS_COMPLETED = 'completed'
ORDER_STATUS_CANCELLED = 'cancelled'

ORDER_STATUSES = (
    (ORDER_STATUS_NEW, _('New order')),
    (ORDER_STATUS_IN_PROGRESS, _('In progress')),
    (ORDER_STATUS_COMPLETED, _('Completed')),
    (ORDER_STATUS_CANCELLED, _('Cancelled')),
)

PAYMENT_METHOD_CASH = 'cash'
PAYMENT_METHOD_PRIVAT24 = 'privat24'
PAYMENT_METHOD_CASH_ON_DELIVERY = 'cash_on_delivery'
PAYMENT_METHOD_CASHLESS_PAYMENT = 'cashless_payment'

PAYMENT_METHODS = (
    (PAYMENT_METHOD_CASH, _('Cash payment')),
    (PAYMENT_METHOD_PRIVAT24, _('Privat 24')),
    (PAYMENT_METHOD_CASH_ON_DELIVERY, _('Cash on delivery')),
    (PAYMENT_METHOD_CASHLESS_PAYMENT, _('Cashless payment')),
)





from django.apps import AppConfig
from django.utils.translation import ugettext_lazy as _


class OrdersConfig(AppConfig):
    name = 'orders'
    verbose_name = _("Orders")


default_app_config = 'orders.OrdersConfig'




from django.db import transaction
from django.shortcuts import redirect, render

from cart.lib import get_cart

from orders.utils import send_new_order_notifications
from orders.forms import CheckoutForm


@transaction.atomic
def checkout(request):

    user = request.user
    initial = {'mobile': '+380'}

    if user.is_authenticated:
        initial.update({
            'first_name': user.first_name,
            'last_name': user.last_name,
            'email': user.email,
        })

    form = CheckoutForm(
        initial=initial,
        data=request.POST or None
    )

    if request.method == 'POST' and form.is_valid():

        cart = get_cart(request)

        order = form.save(commit=False)

        order.delivery = form.fields['delivery'].get_delivery_method()
        order.address = form.fields['delivery'].get_address()

        if user.is_authenticated:
            order.user = request.user

        order.save()

        for item in cart.items:
            order.items.create(
                product_id=item.id,
                qty=item.qty,
                **item.price_values)

        cart.clear()

        send_new_order_notifications(request, order)

        return redirect('home')

    return render(request, 'orders/checkout.html', {'form': form})




from django.db import models
from django.conf import settings
from django.utils.translation import ugettext_lazy as _

from exchange.models import format_printable_price, MultiCurrencyPrice
from delivery.models import DeliveryMethodField

from orders.constants import (
    PAYMENT_METHODS,
    ORDER_STATUSES,
    ORDER_STATUS_NEW
)


class Order(models.Model):

    user = models.ForeignKey(
        settings.AUTH_USER_MODEL, related_name='orders',
        verbose_name=_('Owner'), null=True, blank=True,
        on_delete=models.SET_NULL)

    status = models.CharField(
        _('Status'),
        max_length=100,
        choices=ORDER_STATUSES,
        default=ORDER_STATUS_NEW)

    payment_method = models.CharField(
        _('Payment method'),
        max_length=100,
        choices=PAYMENT_METHODS)

    delivery = DeliveryMethodField()

    first_name = models.CharField(_('First name'), max_length=255)

    last_name = models.CharField(_('Last name'), max_length=255)

    middle_name = models.CharField(
        _('Middle name'), max_length=255, blank=True)

    address = models.CharField(_('Address'), max_length=255, blank=True)

    mobile = models.CharField(_('Mobile number'), max_length=255)

    created = models.DateTimeField(
        _('Date created'), auto_now_add=True, editable=False)

    comment = models.TextField(_('Comment'), max_length=1000, blank=True)

    def __str__(self):
        return self.printable_name

    @property
    def printable_name(self):
        return '{} #{}'.format(_('Order'), self.id)

    @property
    def full_name(self):
        return '{} {} {}'.format(
            self.last_name, self.first_name, self.middle_name)

    @property
    def total(self):
        return sum([i.subtotal for i in self.items.all()])

    @property
    def printable_total(self):
        return format_printable_price(self.total)

    @property
    def delivery_method(self):
        return self.delivery.name

    class Meta:
        verbose_name = _('Order')
        verbose_name_plural = _('Orders')


class OrderedProduct(MultiCurrencyPrice):

    order = models.ForeignKey(
        Order,
        verbose_name=_('Order'),
        related_name='items',
        on_delete=models.CASCADE)

    product = models.ForeignKey(
        'products.Product',
        verbose_name=_('Product'),
        related_name='order_items',
        on_delete=models.CASCADE)

    qty = models.PositiveIntegerField(_('Quantity'), default=1)

    def __str__(self):
        return str(self.product)

    @property
    def subtotal(self):
        return self.price * self.qty

    def printable_subtotal(self):
        return format_printable_price(self.subtotal)

    class Meta:
        verbose_name = _('Ordered product')
        verbose_name_plural = _('Ordered products')




from django.urls import path, include

from django.conf.urls.i18n import i18n_patterns

from orders import views


app_name = 'orders'


urlpatterns = [
    path('checkout/', views.checkout, name='checkout')
]


app_urls = i18n_patterns(

    path('orders/', include((urlpatterns, app_name))),

    path('delivery/', include('delivery.urls'))

)



# Generated by Django 3.0.10 on 2020-11-26 20:31

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('orders', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='order',
            name='user',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='orders', to=settings.AUTH_USER_MODEL, verbose_name='Owner'),
        ),
    ]
